// Jean Luc Biellmann - contact@alsatux.com

"use strict";

var _TabMain = {
	init: function () {
		_DOM.listen('main_prev_year', 'click', _TabMain.slot.click_prev_year);
		_DOM.listen('main_next_year', 'click', _TabMain.slot.click_next_year);
		_DOM.listen('main_goto_today', 'click', _TabMain.slot.click_today);
		_DOM.listen('main_export', 'click', _TabMain.slot.click_export);
		// no onchange() on input_keywords_filter please (loop) ...
		_DOM.listen('button_filter_submit', 'click', _TabMain.slot.filter_submit);
		_DOM.listen('button_filter_reset', 'click', _TabMain.slot.filter_reset);
		_DOM.listen('button_filter_open', 'click', _TabFilter.slot.show);
	},
	slot: {
		click_prev_year: function (e) {
			return _Lock.action_with_spinner(e, function (e) {
				_TabCal.current_year--;
				_TabCal.redraw();
			});
		},
		click_next_year: function (e) {
			return _Lock.action_with_spinner(e, function (e) {
				_TabCal.current_year++;
				_TabCal.redraw();
			});
		},
		click_today: function (e) {
			var now = new Date();
			if (_TabCal.current_year!=now.getFullYear()) {
				return _Lock.action_with_spinner(e, function (e) {
					_TabMain.scroll_today();
				});
			} else {
				return _Lock.action(e, function (e) {
					_TabMain.scroll_today();
				});
			}
		},
		click_export: function (e) {
			return _Lock.action(e, function (e) {
				_TabMain.export();
			});
		},
		filter_submit: function (e) {
			_TabMain.filter_events(1);
		},
		filter_reset: function (e) {
			_DOM.id('input_keywords_filter').value = '';
			_TabMain.remove_events_filtered();
		}
	},
	filter_clean_keywords: function (keywords) {
		var res = [];
		var re = /\s+/;
		var kws = keywords.split(re);
		var kws2 = [];
		for (var i=0;i<kws.length;i++) {
			var kw = kws[i].trim();
			if (kw.length) {
				kws2.push(kw);
			}
		}
		return kws2;
	},
	filter_events: function (add_keywords_as_new_filter) {
		var keywords = _DOM.id('input_keywords_filter').value;
		var kws2 = _TabMain.filter_clean_keywords(keywords);
		if (kws2.length) {
			if (add_keywords_as_new_filter) {
				var filter = {
					'seq': _FilterDB.next_seq(),
					'value': kws2.join(' ')
				};
				try {
					_FilterDB.chk(filter);
					_FilterDB.add(filter);
				} catch (err) {
					throw err;
				}
				_File.save();
			}
			_TabMain.add_events_filtered(kws2);
		} else {
			_TabMain.remove_events_filtered();
		}			
	},
	add_events_filtered: function (kws2) {
		_TabMain.remove_events_filtered();
		var pattern = '(' + kws2.join('|') + ')';
		var re2 = new RegExp('.*' + pattern + '.*','gi');
		var days = _DOM.id('cal').querySelectorAll('div.day');
		for (var i=0;i<days.length;i++) {
			var day = days[i];
			var events = day.querySelectorAll('div.event');
			if (events.length==0) {	
				day.classList.add('filtered');
			} else {	
				var nb_filtered = 0;
				for (var j=0;j<events.length;j++) {
					var event = events[j];
					var found = false;
					/* search into title */
					var title = event.querySelector('span.title');
					if (title!=null) {
						if (title.innerHTML.match(re2))
							found = true;
					}
					/* search into category */
					var cat = event.querySelector('span.cat');
					if (cat!=null) {
						if (cat.innerHTML.match(re2))
							found = true;
					}
					/* search into description */
					var seq = event.getAttribute('data-event');
					var data = _EventDB.db[seq];
					if (data.description) {
						if (data.description.match(re2))
							found = true;
					}						
					/* not found -> add class filtered */
					if (!found) {
						event.classList.add('filtered');
						nb_filtered++;
					}
				}
				/* all event filtered for the current day -> add class filtered */
				if (nb_filtered==events.length)
					day.classList.add('filtered');
			}
		}			
	},
	remove_events_filtered: function () {
		var objs = _DOM.id('cal').querySelectorAll('.filtered');
		for (var i=0;i<objs.length;i++) {
			objs[i].classList.remove('filtered');
		}
	},
	scroll_today: function () {
		var now = new Date();
		if (_TabCal.current_year!=now.getFullYear()) {
			_TabCal.current_year = now.getFullYear();
			_TabCal.redraw();
		} else {
			var div_today = document.querySelectorAll('div.today')[0];
			if (div_today.id!='day_' + now.to_yyyymmdd('-'))
				div_today.classList.remove('today');
		}
		var div_now = _DOM.id('day_' + now.to_yyyymmdd('-'));
		_DOM.id('cal').parentNode.scrollTop = div_now.offsetTop;
		if (!div_now.classList.contains('today'))
			div_now.classList.add('today');
	},
	export: function () {
		var xml_string = _XML.write();
		if (!window.URL || !window.URL.createObjectURL)
			return alert(tr('Warning: your browser doesn\'t support XML file creation on the fly ! Please use Mozilla Firefox !'));
		var blob = new Blob([xml_string], {type:'text/plain;charset=utf-8'});
		var a = document.createElementNS("http://www.w3.org/1999/xhtml", "a");
		a.href = window.URL.createObjectURL(blob);
		a.download = (new Date()).to_yyyymmddhhiiss() + '.xml';
		var event = document.createEvent("MouseEvents");
		event.initMouseEvent('click', true, false, window, 0, 0, 0, 0, 0, false, false, false, false, 0, null);
		a.dispatchEvent(event);
	}
};

